from setuptools import setup

setup(
    name='delegator',
    author='Hugo Bastien',
    author_email='hugobast@gmail.com',
    url='https://github.com/hugobast/delegator',
    version='0.0.3',
    packages=['delegator'],
    license='MIT License',
    long_description=open('README.txt').read(),
)
